# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'image_dialog_ui.ui'
#
# Created by: PyQt5 UI code generator 5.8.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_ImageDialog(object):
    def setupUi(self, ImageDialog):
        ImageDialog.setObjectName("ImageDialog")
        ImageDialog.resize(790, 670)
        self.windowTitle = QtWidgets.QLabel(ImageDialog)
        self.windowTitle.setGeometry(QtCore.QRect(0, 0, 791, 31))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.windowTitle.setFont(font)
        self.windowTitle.setAutoFillBackground(True)
        self.windowTitle.setAlignment(QtCore.Qt.AlignCenter)
        self.windowTitle.setObjectName("windowTitle")
        self.image = QtWidgets.QWidget(ImageDialog)
        self.image.setGeometry(QtCore.QRect(0, 140, 791, 531))
        self.image.setAutoFillBackground(True)
        self.image.setObjectName("image")
        self.btn_expose = QtWidgets.QPushButton(ImageDialog)
        self.btn_expose.setGeometry(QtCore.QRect(10, 50, 81, 81))
        self.btn_expose.setObjectName("btn_expose")
        self.btn_crosshair = QtWidgets.QPushButton(ImageDialog)
        self.btn_crosshair.setGeometry(QtCore.QRect(340, 60, 81, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_crosshair.setFont(font)
        self.btn_crosshair.setObjectName("btn_crosshair")
        self.groupBox_2 = QtWidgets.QGroupBox(ImageDialog)
        self.groupBox_2.setGeometry(QtCore.QRect(710, 40, 71, 91))
        self.groupBox_2.setObjectName("groupBox_2")
        self.btn_strechLow = QtWidgets.QRadioButton(self.groupBox_2)
        self.btn_strechLow.setGeometry(QtCore.QRect(10, 20, 61, 21))
        self.btn_strechLow.setObjectName("btn_strechLow")
        self.btn_strechMid = QtWidgets.QRadioButton(self.groupBox_2)
        self.btn_strechMid.setGeometry(QtCore.QRect(10, 40, 61, 21))
        self.btn_strechMid.setObjectName("btn_strechMid")
        self.btn_strechHigh = QtWidgets.QRadioButton(self.groupBox_2)
        self.btn_strechHigh.setGeometry(QtCore.QRect(10, 60, 61, 21))
        self.btn_strechHigh.setObjectName("btn_strechHigh")
        self.groupBox_3 = QtWidgets.QGroupBox(ImageDialog)
        self.groupBox_3.setGeometry(QtCore.QRect(630, 40, 71, 91))
        self.groupBox_3.setObjectName("groupBox_3")
        self.btn_size25 = QtWidgets.QRadioButton(self.groupBox_3)
        self.btn_size25.setGeometry(QtCore.QRect(10, 20, 51, 21))
        self.btn_size25.setObjectName("btn_size25")
        self.btn_size50 = QtWidgets.QRadioButton(self.groupBox_3)
        self.btn_size50.setGeometry(QtCore.QRect(10, 40, 51, 21))
        self.btn_size50.setObjectName("btn_size50")
        self.btn_size100 = QtWidgets.QRadioButton(self.groupBox_3)
        self.btn_size100.setGeometry(QtCore.QRect(10, 60, 51, 21))
        self.btn_size100.setObjectName("btn_size100")
        self.btn_crosshair_2 = QtWidgets.QPushButton(ImageDialog)
        self.btn_crosshair_2.setGeometry(QtCore.QRect(340, 100, 81, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_crosshair_2.setFont(font)
        self.btn_crosshair_2.setObjectName("btn_crosshair_2")
        self.groupBox_4 = QtWidgets.QGroupBox(ImageDialog)
        self.groupBox_4.setGeometry(QtCore.QRect(540, 40, 81, 91))
        self.groupBox_4.setObjectName("groupBox_4")
        self.btn_colorGray = QtWidgets.QRadioButton(self.groupBox_4)
        self.btn_colorGray.setGeometry(QtCore.QRect(10, 20, 61, 21))
        self.btn_colorGray.setObjectName("btn_colorGray")
        self.btn_colorCool = QtWidgets.QRadioButton(self.groupBox_4)
        self.btn_colorCool.setGeometry(QtCore.QRect(10, 40, 61, 21))
        self.btn_colorCool.setObjectName("btn_colorCool")
        self.btn_colorRainbow = QtWidgets.QRadioButton(self.groupBox_4)
        self.btn_colorRainbow.setGeometry(QtCore.QRect(10, 60, 71, 21))
        self.btn_colorRainbow.setObjectName("btn_colorRainbow")
        self.btn_stasrContExposures = QtWidgets.QPushButton(ImageDialog)
        self.btn_stasrContExposures.setGeometry(QtCore.QRect(100, 50, 81, 81))
        self.btn_stasrContExposures.setObjectName("btn_stasrContExposures")
        self.btn_stopContExposures = QtWidgets.QPushButton(ImageDialog)
        self.btn_stopContExposures.setGeometry(QtCore.QRect(190, 50, 81, 81))
        self.btn_stopContExposures.setObjectName("btn_stopContExposures")
        self.btn_connectCamPS = QtWidgets.QPushButton(ImageDialog)
        self.btn_connectCamPS.setGeometry(QtCore.QRect(440, 60, 81, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_connectCamPS.setFont(font)
        self.btn_connectCamPS.setObjectName("btn_connectCamPS")
        self.btn_disconnectCamPS = QtWidgets.QPushButton(ImageDialog)
        self.btn_disconnectCamPS.setGeometry(QtCore.QRect(440, 100, 81, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_disconnectCamPS.setFont(font)
        self.btn_disconnectCamPS.setObjectName("btn_disconnectCamPS")
        self.line_13 = QtWidgets.QFrame(ImageDialog)
        self.line_13.setGeometry(QtCore.QRect(490, 40, 31, 21))
        self.line_13.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_13.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_13.setObjectName("line_13")
        self.label_81 = QtWidgets.QLabel(ImageDialog)
        self.label_81.setGeometry(QtCore.QRect(440, 40, 51, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_81.setFont(font)
        self.label_81.setObjectName("label_81")
        self.label_82 = QtWidgets.QLabel(ImageDialog)
        self.label_82.setGeometry(QtCore.QRect(340, 40, 51, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_82.setFont(font)
        self.label_82.setObjectName("label_82")
        self.line_14 = QtWidgets.QFrame(ImageDialog)
        self.line_14.setGeometry(QtCore.QRect(390, 40, 31, 21))
        self.line_14.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_14.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_14.setObjectName("line_14")

        self.retranslateUi(ImageDialog)
        QtCore.QMetaObject.connectSlotsByName(ImageDialog)

    def retranslateUi(self, ImageDialog):
        _translate = QtCore.QCoreApplication.translate
        ImageDialog.setWindowTitle(_translate("ImageDialog", "Form"))
        self.windowTitle.setText(_translate("ImageDialog", "Imaging with ASCOM Camera"))
        self.btn_expose.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Single exposure</p></body></html>"))
        self.btn_expose.setText(_translate("ImageDialog", "Expose\n"
"single\n"
"Shot"))
        self.btn_crosshair.setText(_translate("ImageDialog", "Crosshairs"))
        self.groupBox_2.setTitle(_translate("ImageDialog", "Strech"))
        self.btn_strechLow.setText(_translate("ImageDialog", "Low"))
        self.btn_strechMid.setText(_translate("ImageDialog", "Mid"))
        self.btn_strechHigh.setText(_translate("ImageDialog", "High"))
        self.groupBox_3.setTitle(_translate("ImageDialog", "Zoom"))
        self.btn_size25.setText(_translate("ImageDialog", "25%"))
        self.btn_size50.setText(_translate("ImageDialog", "50%"))
        self.btn_size100.setText(_translate("ImageDialog", "100%"))
        self.btn_crosshair_2.setText(_translate("ImageDialog", "Solve"))
        self.groupBox_4.setTitle(_translate("ImageDialog", "Colors"))
        self.btn_colorGray.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Color scheme black /white</p></body></html>"))
        self.btn_colorGray.setText(_translate("ImageDialog", "Gray"))
        self.btn_colorCool.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Color scheme red/blue</p></body></html>"))
        self.btn_colorCool.setText(_translate("ImageDialog", "Cool"))
        self.btn_colorRainbow.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Color scheme rainbow</p></body></html>"))
        self.btn_colorRainbow.setText(_translate("ImageDialog", "Rainbow"))
        self.btn_stasrContExposures.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Starting taking continously exposures</p></body></html>"))
        self.btn_stasrContExposures.setText(_translate("ImageDialog", "Start\n"
"cont.\n"
"Exposures"))
        self.btn_stopContExposures.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Stopping taking continously exposures</p></body></html>"))
        self.btn_stopContExposures.setText(_translate("ImageDialog", "Stop\n"
"cont.\n"
"Exposures"))
        self.btn_connectCamPS.setText(_translate("ImageDialog", "Connect"))
        self.btn_disconnectCamPS.setText(_translate("ImageDialog", "Disconnect"))
        self.label_81.setText(_translate("ImageDialog", "Camera"))
        self.label_82.setText(_translate("ImageDialog", "Views"))

